"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prompts_1 = __importDefault(require("prompts"));
const node_path_1 = __importDefault(require("node:path"));
const promises_1 = __importDefault(require("node:fs/promises"));
const node_child_process_1 = require("node:child_process");
function required(s) {
    if (s.length === 0) {
        return 'This value is required';
    }
    return true;
}
// https://github.com/vitejs/vite/blob/76082e3d3033b09b02b6db64de6e36942593c753/packages/create-vite/src/index.ts#L557
function isValidPackageName(projectName) {
    return /^(?:@[a-z\d\-*~][a-z\d\-*._~]*\/)?[a-z\d\-~][a-z\d\-._~]*$/.test(projectName) || 'Invalid package name';
}
function askConfiguration() {
    return (0, prompts_1.default)([
        {
            type: 'text',
            name: 'name',
            message: 'Language name',
            validate: required,
        },
        {
            type: 'text',
            name: 'packageName',
            message: 'Package name',
            validate: isValidPackageName,
            initial: (_, answers) => `my-dynamic-lang-${answers.name}`,
        },
        {
            type: 'text',
            name: 'treeSitterPackage',
            message: 'Tree-sitter package to use',
            validate: isValidPackageName,
            initial: (_, answers) => `tree-sitter-${answers.name}`,
        },
        {
            type: 'list',
            name: 'extensions',
            message: 'File extensions used by the language, comma separated',
            separator: ',',
            validate: required,
        },
        {
            type: 'text',
            name: 'expandoChar',
            message: 'Expando char used in pattern',
            initial: '$',
            validate: (value) => {
                return value.length === 1 ? true : 'Expando char must be a single character';
            }
        },
        {
            type: 'confirm',
            name: 'includeDotFiles',
            message: 'Include gitignore and npm publish files?',
            initial: true,
        }
    ], {
        onCancel: () => {
            process.exit(1);
        }
    });
}
function copyTemplate(targetDir, includeDotFiles) {
    const templateDir = node_path_1.default.join(__dirname, 'template');
    return promises_1.default.cp(templateDir, targetDir, {
        recursive: true, // Copy all files and folders
        // includes hidden files if `includeDotFiles` is true
        filter: (src) => {
            const basename = node_path_1.default.basename(src);
            return includeDotFiles || !basename.startsWith('.');
        }
    });
}
async function renameFiles(dir, answer) {
    const name = {
        $$PACKAGE_NAME$$: answer.packageName,
        $$NAME$$: answer.name,
        $$TREE_SITTER_PACKAGE$$: answer.treeSitterPackage,
        $$EXTENSIONS$$: JSON.stringify(answer.extensions),
        $$EXPANDO_CHAR$$: answer.expandoChar,
    };
    for (const file of await promises_1.default.readdir(dir)) {
        const filePath = node_path_1.default.join(dir, file);
        const stats = await promises_1.default.stat(filePath);
        if (stats.isDirectory()) {
            renameFiles(filePath, answer);
        }
        else {
            const content = await promises_1.default.readFile(filePath, 'utf-8');
            const newContent = content.replace(/(\$\$[A-Z_]+\$\$)/g, (match) => {
                return name[match] || match;
            });
            await promises_1.default.writeFile(filePath, newContent);
        }
    }
}
function installTreeSitterPackage(cwd, answer) {
    console.log('Installing tree-sitter package...');
    (0, node_child_process_1.execSync)(`pnpm install ${answer.treeSitterPackage} --save-dev --save-exact`, {
        cwd,
    });
    console.log('Copying source code...');
    (0, node_child_process_1.execSync)('pnpm run source', { cwd });
    console.log('Compiling');
    (0, node_child_process_1.execSync)('pnpm run build', { cwd });
}
async function main() {
    let cwd = process.cwd();
    if (process.argv.length > 2) {
        const targetDir = process.argv[2];
        cwd = node_path_1.default.join(cwd, targetDir);
    }
    const config = await askConfiguration();
    await copyTemplate(cwd, config.includeDotFiles);
    await renameFiles(cwd, config);
    installTreeSitterPackage(cwd, config);
}
main();
